@@ -29,7 +29,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <gconf/gconf-client.h>
 #include <unique/uniqueapp.h>
 
 #include <bluetooth-client.h>
@@ -48,8 +47,8 @@ static guint num_adapters_powered = 0;
 static gboolean show_icon_pref = TRUE;
 static gboolean discover_lock = FALSE;
 
-#define PREF_DIR		"/apps/bluetooth-manager"
-#define PREF_SHOW_ICON		PREF_DIR "/show_icon"
+#define SCHEMA_NAME		"org.gnome.Bluetooth"
+#define PREF_SHOW_ICON		"show-icon"
 
 #define KEYBOARD_PREFS		"gnome-keyboard-properties"
 #define MOUSE_PREFS		"gnome-mouse-properties"
@@ -63,7 +62,7 @@ enum {
 	DISCONNECTING
 };
 
-static GConfClient* gconf;
+static GSettings *settings;
 static BluetoothKillswitch *killswitch = NULL;
 
 static GtkBuilder *xml = NULL;
@@ -79,6 +78,7 @@ void wizard_callback(GObject *widget, gpointer user_data);
 void sendto_callback(GObject *widget, gpointer user_data);
 
 static void action_set_bold (GtkUIManager *manager, GtkAction *action, const char *path);
+static void update_icon_visibility (void);
 
 void quit_callback(GObject *widget, gpointer user_data)
 {
@@ -114,9 +114,16 @@ mount_finish_cb (GObject *source_object,
 
 	if (g_file_mount_enclosing_volume_finish (G_FILE (source_object),
 						  res, &error) == FALSE) {
-		g_printerr ("Failed to mount OBEX volume: %s", error->message);
-		g_error_free (error);
-		return;
+		/* Ignore "already mounted" error */
+		if (error->domain == G_IO_ERROR &&
+		    error->code == G_IO_ERROR_ALREADY_MOUNTED) {
+			g_error_free (error);
+			error = NULL;
+		} else {
+			g_printerr ("Failed to mount OBEX volume: %s", error->message);
+			g_error_free (error);
+			return;
+		}
 	}
 
 	uri = g_file_get_uri (G_FILE (source_object));
@@ -255,6 +262,45 @@ void wizard_callback(GObject *widget, gpointer user_data)
 		g_printerr("Couldn't execute command: %s\n", command);
 }
 
+static gboolean
+set_powered_foreach (GtkTreeModel *model,
+		     GtkTreePath  *path,
+		     GtkTreeIter  *iter,
+		     gpointer      data)
+{
+	DBusGProxy *proxy = NULL;
+	GValue value = { 0, };
+
+	gtk_tree_model_get (model, iter,
+			    BLUETOOTH_COLUMN_PROXY, &proxy, -1);
+	if (proxy == NULL)
+		return FALSE;
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+
+	dbus_g_proxy_call_no_reply (proxy, "SetProperty",
+				    G_TYPE_STRING, "Powered",
+				    G_TYPE_VALUE, &value,
+				    G_TYPE_INVALID,
+				    G_TYPE_INVALID);
+
+	g_value_unset (&value);
+	g_object_unref (proxy);
+
+	return FALSE;
+}
+
+static void
+bluetooth_set_adapter_powered (void)
+{
+	GtkTreeModel *adapters;
+
+	adapters = bluetooth_client_get_adapter_model (client);
+	gtk_tree_model_foreach (adapters, set_powered_foreach, NULL);
+	g_object_unref (adapters);
+}
+
 void bluetooth_status_callback (GObject *widget, gpointer user_data)
 {
 	GObject *object;
@@ -266,6 +312,7 @@ void bluetooth_status_callback (GObject *widget, gpointer user_data)
 	bluetooth_killswitch_set_state (killswitch,
 					active ? KILLSWITCH_STATE_UNBLOCKED : KILLSWITCH_STATE_SOFT_BLOCKED);
 	g_object_set_data (object, "bt-active", GINT_TO_POINTER (active));
+	bluetooth_set_adapter_powered ();
 }
 
 void
@@ -325,6 +372,7 @@ killswitch_state_changed (BluetoothKillswitch *killswitch, KillswitchState state
 	if (state == KILLSWITCH_STATE_NO_ADAPTER) {
 		object = gtk_builder_get_object (xml, "bluetooth-applet-popup");
 		gtk_menu_popdown (GTK_MENU (object));
+		update_icon_visibility ();
 		return;
 	}
 
@@ -359,6 +407,8 @@ killswitch_state_changed (BluetoothKillswitch *killswitch, KillswitchState state
 
 	object = gtk_builder_get_object (xml, "bluetooth-applet-ui-manager");
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (object));
+
+	update_icon_visibility ();
 }
 
 static GtkWidget *create_popupmenu(void)
@@ -464,6 +514,7 @@ update_discoverability (GtkTreeIter *iter)
 	object = gtk_builder_get_object (xml, "discoverable");
 
 	if (iter == NULL) {
+		discover_lock = FALSE;
 		gtk_action_set_visible (GTK_ACTION (object), FALSE);
 		return;
 	}
@@ -1017,20 +1068,13 @@ static void device_removed(GtkTreeModel *model,
 	device_changed (model, path, NULL, user_data);
 }
 
-static void gconf_callback(GConfClient *client, guint cnxn_id,
-					GConfEntry *entry, gpointer user_data)
+static void
+show_icon_changed (GSettings *settings,
+		   const char *key,
+		   gpointer   user_data)
 {
-	GConfValue *value;
-
-	value = gconf_entry_get_value(entry);
-	if (value == NULL)
-		return;
-
-	if (g_str_equal(entry->key, PREF_SHOW_ICON) == TRUE) {
-		show_icon_pref = gconf_value_get_bool(value);
-		update_icon_visibility();
-		return;
-	}
+	show_icon_pref = g_settings_get_boolean (settings, PREF_SHOW_ICON);
+	update_icon_visibility();
 }
 
 static GOptionEntry options[] = {
@@ -1043,7 +1087,6 @@ int main(int argc, char *argv[])
 	UniqueApp *app;
 	GtkStatusIcon *statusicon;
 	GtkWidget *menu;
-	GConfValue *value;
 	GOptionContext *context;
 	GError *error = NULL;
 
@@ -1096,27 +1139,24 @@ int main(int argc, char *argv[])
 			 G_CALLBACK(device_removed), NULL);
 	g_signal_connect (G_OBJECT (devices_model), "row-changed",
 			  G_CALLBACK (device_changed), NULL);
-	/* Set the default */
+
+	/* Set the default adapter */
 	device_changed (devices_model, NULL, NULL, NULL);
 	if (bluetooth_killswitch_has_killswitches (killswitch) != FALSE) {
 		killswitch_state_changed (killswitch,
 					  bluetooth_killswitch_get_state (killswitch));
 	}
 
-	gconf = gconf_client_get_default();
+	/* Make sure all the unblocked adapters are powered,
+	 * so as to avoid seeing unpowered, but unblocked
+	 * devices */
+	bluetooth_set_adapter_powered ();
 
-	value = gconf_client_get (gconf, PREF_SHOW_ICON, NULL);
-	if (value == NULL) {
-		show_icon_pref = TRUE;
-	} else {
-		show_icon_pref = gconf_value_get_bool (value);
-		gconf_value_free (value);
-	}
+	settings = g_settings_new (SCHEMA_NAME);
+	show_icon_pref = g_settings_get_boolean (settings, PREF_SHOW_ICON);
 
-	gconf_client_add_dir(gconf, PREF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
-
-	gconf_client_notify_add(gconf, PREF_DIR,
-					gconf_callback, NULL, NULL, NULL);
+	g_signal_connect (G_OBJECT (settings), "changed::" PREF_SHOW_ICON,
+			  G_CALLBACK (show_icon_changed), NULL);
 
 	statusicon = init_notification();
 
@@ -1133,7 +1173,7 @@ int main(int argc, char *argv[])
 
 	gtk_widget_destroy(menu);
 
-	g_object_unref(gconf);
+	g_object_unref(settings);
 
 	cleanup_agents();
 
