@@ -18,6 +18,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:bluetooth-chooser-button
+ * @short_description: a Bluetooth chooser button
+ * @stability: Stable
+ * @include: bluetooth-plugin.h
+ *
+ * A button used to select Bluetooth devices which will pop-up a
+ * #BluetoothChooser widget inside a dialogue when clicked.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -210,6 +220,7 @@ bluetooth_chooser_button_clicked (GtkButton *widget)
 	g_signal_emit (G_OBJECT (button),
 		       signals[CHOOSER_CREATED],
 		       0, button->chooser);
+	g_object_set (G_OBJECT (button->chooser), "device-selected", button->bdaddr, NULL);
 	gtk_container_set_border_width (GTK_CONTAINER(button->chooser), 5);
 	gtk_widget_show (button->chooser);
 	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (button->dialog))), button->chooser);
@@ -378,7 +389,9 @@ bluetooth_chooser_button_init (BluetoothChooserButton *button)
 /**
  * bluetooth_chooser_button_new:
  *
- * Return value: a #BluetoothChooserButton
+ * Returns a new #BluetoothChooserButton widget.
+ *
+ * Return value: a #BluetoothChooserButton widget.
  **/
 GtkWidget *
 bluetooth_chooser_button_new (void)
@@ -392,6 +405,8 @@ bluetooth_chooser_button_new (void)
  * bluetooth_chooser_button_available:
  * @button: a #BluetoothChooserButton
  *
+ * Returns whether there is a powered Bluetooth adapter.
+ *
  * Return value: %TRUE if there is a powered Bluetooth adapter available, and the button should be sensitive.
  **/
 gboolean
