@@ -21,6 +21,16 @@
  *
  */
 
+/**
+ * SECTION:bluetooth-client
+ * @short_description: Bluetooth client object
+ * @stability: Stable
+ * @include: bluetooth-client.h
+ *
+ * The #BluetoothClient object is used to query the state of Bluetooth
+ * devices and adapters.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -106,42 +116,46 @@ G_DEFINE_TYPE(BluetoothClient, bluetooth_client, G_TYPE_OBJECT)
  * bluetooth_type_to_string:
  * @type: a #BluetoothType
  *
- * Return value: the string representation of the @type passed. Do not free the return value.
+ * Returns the string representation of the @type passed. Do not free the return value.
+ *
+ * Return value: a string.
  **/
 const gchar *bluetooth_type_to_string(BluetoothType type)
 {
 	switch (type) {
 	case BLUETOOTH_TYPE_ANY:
-		return N_("All types");
+		return _("All types");
 	case BLUETOOTH_TYPE_PHONE:
-		return N_("Phone");
+		return _("Phone");
 	case BLUETOOTH_TYPE_MODEM:
-		return N_("Modem");
+		return _("Modem");
 	case BLUETOOTH_TYPE_COMPUTER:
-		return N_("Computer");
+		return _("Computer");
 	case BLUETOOTH_TYPE_NETWORK:
-		return N_("Network");
+		return _("Network");
 	case BLUETOOTH_TYPE_HEADSET:
 		/* translators: a hands-free headset, a combination of a single speaker with a microphone */
-		return N_("Headset");
+		return _("Headset");
 	case BLUETOOTH_TYPE_HEADPHONES:
-		return N_("Headphones");
+		return _("Headphones");
 	case BLUETOOTH_TYPE_OTHER_AUDIO:
-		return N_("Audio device");
+		return _("Audio device");
 	case BLUETOOTH_TYPE_KEYBOARD:
-		return N_("Keyboard");
+		return _("Keyboard");
 	case BLUETOOTH_TYPE_MOUSE:
-		return N_("Mouse");
+		return _("Mouse");
 	case BLUETOOTH_TYPE_CAMERA:
-		return N_("Camera");
+		return _("Camera");
 	case BLUETOOTH_TYPE_PRINTER:
-		return N_("Printer");
+		return _("Printer");
 	case BLUETOOTH_TYPE_JOYPAD:
-		return N_("Joypad");
+		return _("Joypad");
 	case BLUETOOTH_TYPE_TABLET:
-		return N_("Tablet");
+		return _("Tablet");
+	case BLUETOOTH_TYPE_VIDEO:
+		return _("Video device");
 	default:
-		return N_("Unknown");
+		return _("Unknown");
 	}
 }
 
@@ -149,7 +163,9 @@ const gchar *bluetooth_type_to_string(BluetoothType type)
  * bluetooth_verify_address:
  * @bdaddr: a string representing a Bluetooth address
  *
- * Return value: Whether the string is a valid Bluetooth address. This does not contact the device in any way.
+ * Returns whether the string is a valid Bluetooth address. This does not contact the device in any way.
+ *
+ * Return value: %TRUE if the address is valid, %FALSE if not.
  **/
 gboolean
 bluetooth_verify_address (const char *bdaddr)
@@ -209,6 +225,10 @@ bluetooth_class_to_type (guint32 class)
 			return BLUETOOTH_TYPE_HEADSET;
 		case 0x06:
 			return BLUETOOTH_TYPE_HEADPHONES;
+		case 0x0b: /* VCR */
+		case 0x0c: /* Video Camera */
+		case 0x0d: /* Camcorder */
+			return BLUETOOTH_TYPE_VIDEO;
 		default:
 			return BLUETOOTH_TYPE_OTHER_AUDIO;
 		}
@@ -297,6 +317,23 @@ static gboolean compare_path(GtkTreeStore *store,
 	return found;
 }
 
+static gboolean
+compare_address (GtkTreeStore *store,
+		 GtkTreeIter *iter,
+		 gpointer user_data)
+{
+	const char *address = user_data;
+	char *tmp_address;
+	gboolean found = FALSE;
+
+	gtk_tree_model_get (GTK_TREE_MODEL(store), iter,
+			    BLUETOOTH_COLUMN_ADDRESS, &tmp_address, -1);
+	found = g_str_equal (address, tmp_address);
+	g_free (tmp_address);
+
+	return found;
+}
+
 static gboolean
 get_iter_from_path (GtkTreeStore *store,
 		    GtkTreeIter *iter,
@@ -314,6 +351,20 @@ get_iter_from_proxy(GtkTreeStore *store,
 			   (gpointer) dbus_g_proxy_get_path (proxy));
 }
 
+static gboolean
+get_iter_from_address (GtkTreeStore *store,
+		       GtkTreeIter  *iter,
+		       const char   *address,
+		       DBusGProxy   *adapter)
+{
+	GtkTreeIter parent_iter;
+
+	if (get_iter_from_proxy (store, &parent_iter, adapter) == FALSE)
+		return FALSE;
+
+	return iter_search (store, iter, &parent_iter, compare_address, (gpointer) address);
+}
+
 static void
 device_services_changed (DBusGProxy *iface, const char *property,
 			 GValue *value, gpointer user_data)
@@ -509,6 +560,8 @@ uuid16_to_string (guint uuid16, const char *uuid)
 		return NULL;
 	case 0x1201:
 		return "GenericNetworking";
+	case 0x1303:
+		return "VideoSource";
 	case 0x8e771303:
 	case 0x8e771301:
 		return "SEMC HLA";
@@ -524,7 +577,9 @@ uuid16_to_string (guint uuid16, const char *uuid)
  * bluetooth_uuid_to_string:
  * @uuid: a string representing a Bluetooth UUID
  *
- * Return value: a string representing a human-readable (but not usable for display to users) version of the @uuid. Do not free the return value.
+ * Returns a string representing a human-readable (but not usable for display to users) version of the @uuid. Do not free the return value.
+ *
+ * Return value: a string.
  **/
 const char *
 bluetooth_uuid_to_string (const char *uuid)
@@ -656,7 +711,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 	gboolean paired, trusted, connected;
 	int legacypairing;
 	guint type;
-	gint rssi;
 	GtkTreeIter iter;
 	gboolean cont;
 
@@ -687,9 +741,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 		value = g_hash_table_lookup(hash, "Icon");
 		icon = value ? g_value_get_string(value) : "bluetooth";
 
-		value = g_hash_table_lookup(hash, "RSSI");
-		rssi = value ? g_value_get_int(value) : 0;
-
 		value = g_hash_table_lookup(hash, "Paired");
 		paired = value ? g_value_get_boolean(value) : FALSE;
 
@@ -726,7 +777,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 					BLUETOOTH_COLUMN_NAME, name,
 					BLUETOOTH_COLUMN_TYPE, type,
 					BLUETOOTH_COLUMN_ICON, icon,
-					BLUETOOTH_COLUMN_RSSI, rssi,
 					BLUETOOTH_COLUMN_LEGACYPAIRING, legacypairing,
 					-1);
 			if (uuids != NULL) {
@@ -763,7 +813,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 				BLUETOOTH_COLUMN_NAME, name,
 				BLUETOOTH_COLUMN_TYPE, type,
 				BLUETOOTH_COLUMN_ICON, icon,
-				BLUETOOTH_COLUMN_RSSI, rssi,
 				BLUETOOTH_COLUMN_PAIRED, paired,
 				BLUETOOTH_COLUMN_TRUSTED, trusted,
 				BLUETOOTH_COLUMN_CONNECTED, connected,
@@ -860,6 +909,12 @@ static void adapter_changed(DBusGProxy *adapter, const char *property,
 			priv->default_adapter_powered = powered;
 			g_object_notify (G_OBJECT (client), "default-adapter-powered");
 		}
+	} else if (g_str_equal(property, "Discoverable") == TRUE) {
+		gboolean discoverable = g_value_get_boolean(value);
+
+		gtk_tree_store_set(priv->store, &iter,
+				BLUETOOTH_COLUMN_DISCOVERABLE, discoverable, -1);
+		notify = TRUE;
 	}
 
 	if (notify != FALSE) {
@@ -883,7 +938,7 @@ static void adapter_added(DBusGProxy *manager,
 	GHashTable *hash = NULL;
 	GValue *value;
 	const gchar *address, *name;
-	gboolean discovering, powered;
+	gboolean discovering, discoverable, powered;
 
 	DBG("client %p path %s", client, path);
 
@@ -906,10 +961,14 @@ static void adapter_added(DBusGProxy *manager,
 
 		value = g_hash_table_lookup(hash, "Devices");
 		devices = value ? g_value_get_boxed (value) : NULL;
+
+		value = g_hash_table_lookup (hash, "Discoverable");
+		discoverable = value ? g_value_get_boolean (value) : FALSE;
 	} else {
 		address = NULL;
 		name = NULL;
 		discovering = FALSE;
+		discoverable = FALSE;
 		powered = FALSE;
 		devices = NULL;
 	}
@@ -919,6 +978,7 @@ static void adapter_added(DBusGProxy *manager,
 					  BLUETOOTH_COLUMN_ADDRESS, address,
 					  BLUETOOTH_COLUMN_NAME, name,
 					  BLUETOOTH_COLUMN_DISCOVERING, discovering,
+					  BLUETOOTH_COLUMN_DISCOVERABLE, discoverable,
 					  BLUETOOTH_COLUMN_POWERED, powered,
 					  -1);
 
@@ -1086,9 +1146,9 @@ static void bluetooth_client_init(BluetoothClient *client)
 
 	priv->store = gtk_tree_store_new(_BLUETOOTH_NUM_COLUMNS, G_TYPE_OBJECT,
 					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-					 G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INT,
+					 G_TYPE_UINT, G_TYPE_STRING,
 					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
-					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_INT,
+					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_INT,
 					 G_TYPE_BOOLEAN, G_TYPE_HASH_TABLE, G_TYPE_STRV);
 
 	priv->dbus = dbus_g_proxy_new_for_name(connection, DBUS_SERVICE_DBUS,
@@ -1219,7 +1279,9 @@ static void bluetooth_client_class_init(BluetoothClientClass *klass)
 /**
  * bluetooth_client_new:
  *
- * Return value: a reference to the #BluetoothClient singleton. Unref the object when done.
+ * Returns a reference to the #BluetoothClient singleton. Use #g_object_unref() the object when done.
+ *
+ * Return value: a #BluetoothClient object.
  **/
 BluetoothClient *bluetooth_client_new(void)
 {
@@ -1241,7 +1303,9 @@ BluetoothClient *bluetooth_client_new(void)
  * bluetooth_client_get_model:
  * @client: a #BluetoothClient object
  *
- * Return value: an unfiltered #GtkTreeModel representing the adapter and devices available on the system.
+ * Returns an unfiltered #GtkTreeModel representing the adapter and devices available on the system.
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
 {
@@ -1265,7 +1329,9 @@ GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
  * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
  * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
  *
- * Return value: a #GtkTreeModel of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ * Returns a #GtkTreeModel of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *bluetooth_client_get_filter_model (BluetoothClient *client,
 						 GtkTreeModelFilterVisibleFunc func,
@@ -1310,7 +1376,9 @@ static gboolean adapter_filter(GtkTreeModel *model,
  * bluetooth_client_get_adapter_model:
  * @client: a #BluetoothClient object
  *
- * Return value: a filtered #GtkTreeModel with only adapters present.
+ * Returns a filtered #GtkTreeModel with only adapters present.
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
 {
@@ -1325,7 +1393,9 @@ GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
  * @client: a #BluetoothClient object
  * @adapter: a #DBusGProxy of the adapter object, or %NULL to get the default adapter.
  *
- * Return value: a filtered #GtkTreeModel with only devices belonging to the selected adapter listed. Note that the model will follow a specific adapter, and will not follow the default-adapter when %NULL is passed.
+ * Returns a filtered #GtkTreeModel with only devices belonging to the selected adapter listed. Note that the model will follow a specific adapter, and will not follow the default-adapter when %NULL is passed.
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 						 DBusGProxy *adapter)
@@ -1382,11 +1452,15 @@ GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 /**
  * bluetooth_client_get_device_filter_model:
  * @client: a #BluetoothClient object
+ * @adapter: a #DBusGProxy representing a particular adapter, or %NULL for the default adapter.
  * @func: a #GtkTreeModelFilterVisibleFunc
  * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
  * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
  *
- * Return value: a #GtkTreeModel of adapters filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ * Returns a #GtkTreeModel of adapters filtered using the @func, @data and
+ * @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ *
+ * Return value: a #GtkTreeModel object.
  **/
 GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
 		DBusGProxy *adapter, GtkTreeModelFilterVisibleFunc func,
@@ -1410,7 +1484,9 @@ GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
  * bluetooth_client_get_default_adapter:
  * @client: a #BluetoothClient object
  *
- * Return value: a #DBusGProxy object representing the default adapter, or %NULL if no adapters are present.
+ * Returns a #DBusGProxy object representing the default adapter, or %NULL if no adapters are present.
+ *
+ * Return value: a #DBusGProxy object.
  **/
 DBusGProxy *bluetooth_client_get_default_adapter(BluetoothClient *client)
 {
@@ -1499,6 +1575,59 @@ gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
 	return TRUE;
 }
 
+/**
+ * bluetooth_client_set_discoverable:
+ * @client: a #BluetoothClient object
+ * @discoverable: whether the device should be discoverable
+ *
+ * Sets the default adapter's discoverable status.
+ *
+ * Return value: Whether setting the state on the default adapter was successful.
+ **/
+gboolean
+bluetooth_client_set_discoverable (BluetoothClient *client,
+				   gboolean discoverable)
+{
+	DBusGProxy *adapter;
+	GValue disco = { 0 };
+	GValue timeout = { 0 };
+	gboolean ret;
+
+	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
+
+	DBG("client %p", client);
+
+	adapter = bluetooth_client_get_default_adapter (client);
+	if (adapter == NULL)
+		return FALSE;
+
+	g_value_init (&disco, G_TYPE_BOOLEAN);
+	g_value_init (&timeout, G_TYPE_UINT);
+
+	g_value_set_boolean (&disco, discoverable);
+	g_value_set_uint (&timeout, 0);
+
+	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
+				 G_TYPE_STRING, "Discoverable",
+				 G_TYPE_VALUE, &disco,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+	if (ret == FALSE)
+		goto bail;
+
+	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
+				 G_TYPE_STRING, "DiscoverableTimeout",
+				 G_TYPE_VALUE, &timeout,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+
+bail:
+	g_value_unset (&disco);
+	g_value_unset (&timeout);
+
+	g_object_unref(adapter);
+
+	return ret;
+}
+
 typedef struct {
 	BluetoothClientCreateDeviceFunc func;
 	gpointer data;
@@ -1528,12 +1657,16 @@ static void create_device_callback(DBusGProxy *proxy,
 	g_object_unref(proxy);
 }
 
-gboolean bluetooth_client_create_device(BluetoothClient *client,
-			const char *address, const char *agent,
-			BluetoothClientCreateDeviceFunc func, gpointer data)
+gboolean bluetooth_client_create_device (BluetoothClient *client,
+					 const char *address,
+					 const char *agent,
+					 BluetoothClientCreateDeviceFunc func,
+					 gpointer data)
 {
+	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(client);
 	CreateDeviceData *devdata;
 	DBusGProxy *adapter;
+	GtkTreeIter iter;
 
 	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (address != NULL, FALSE);
@@ -1544,6 +1677,28 @@ gboolean bluetooth_client_create_device(BluetoothClient *client,
 	if (adapter == NULL)
 		return FALSE;
 
+	/* Remove the pairing if it already exists, but only for pairings */
+	if (agent != NULL &&
+	    get_iter_from_address(priv->store, &iter, address, adapter) == TRUE) {
+		DBusGProxy *device;
+		gboolean paired;
+		GError *err = NULL;
+
+		gtk_tree_model_get (GTK_TREE_MODEL(priv->store), &iter,
+				    BLUETOOTH_COLUMN_PROXY, &device,
+				    BLUETOOTH_COLUMN_PAIRED, &paired, -1);
+		if (paired != FALSE &&
+		    dbus_g_proxy_call (adapter, "RemoveDevice", &err,
+				       DBUS_TYPE_G_OBJECT_PATH, dbus_g_proxy_get_path (device),
+				       G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
+			g_warning ("Failed to remove device '%s': %s", address,
+				   err->message);
+			g_error_free (err);
+		}
+		if (device != NULL)
+			g_object_unref (device);
+	}
+
 	devdata = g_try_new0(CreateDeviceData, 1);
 	if (devdata == NULL)
 		return FALSE;
@@ -1824,8 +1979,17 @@ gboolean bluetooth_client_disconnect_service (BluetoothClient *client,
 static void
 services_foreach (const char *service, gpointer _value, GString *str)
 {
-	gboolean value = GPOINTER_TO_INT (_value);
-	g_string_append_printf (str, "%s (%s) ", service, value ? "connected" : "not connected");
+	GEnumClass *eclass;
+	GEnumValue *ev;
+	BluetoothStatus status = GPOINTER_TO_INT (_value);
+
+	eclass = g_type_class_ref (BLUETOOTH_TYPE_STATUS);
+	ev = g_enum_get_value (eclass, status);
+	if (ev == NULL)
+		g_warning ("Unknown status value %d", status);
+
+	g_string_append_printf (str, "%s (%s) ", service, ev ? ev->value_nick : "unknown");
+	g_type_class_unref (eclass);
 }
 
 void
@@ -1835,7 +1999,7 @@ bluetooth_client_dump_device (GtkTreeModel *model,
 {
 	DBusGProxy *proxy;
 	char *address, *alias, *name, *icon, **uuids;
-	gboolean is_default, paired, trusted, connected, discovering, powered, is_adapter;
+	gboolean is_default, paired, trusted, connected, discoverable, discovering, powered, is_adapter;
 	GHashTable *services;
 	GtkTreeIter parent;
 	guint type;
@@ -1850,6 +2014,7 @@ bluetooth_client_dump_device (GtkTreeModel *model,
 			    BLUETOOTH_COLUMN_PAIRED, &paired,
 			    BLUETOOTH_COLUMN_TRUSTED, &trusted,
 			    BLUETOOTH_COLUMN_CONNECTED, &connected,
+			    BLUETOOTH_COLUMN_DISCOVERABLE, &discoverable,
 			    BLUETOOTH_COLUMN_DISCOVERING, &discovering,
 			    BLUETOOTH_COLUMN_POWERED, &powered,
 			    BLUETOOTH_COLUMN_SERVICES, &services,
@@ -1863,6 +2028,7 @@ bluetooth_client_dump_device (GtkTreeModel *model,
 		g_print ("Adapter: %s (%s)\n", name, address);
 		if (is_default)
 			g_print ("\tDefault adapter\n");
+		g_print ("\tDiscoverable: %s\n", BOOL_STR (discoverable));
 		if (discovering)
 			g_print ("\tDiscovery in progress\n");
 		g_print ("\t%s\n", powered ? "Is powered" : "Is not powered");
