@@ -89,6 +89,23 @@ event_to_state (guint soft, guint hard)
 		return KILLSWITCH_STATE_UNBLOCKED;
 }
 
+static const char *
+state_to_string (KillswitchState state)
+{
+	switch (state) {
+	case KILLSWITCH_STATE_NO_ADAPTER:
+		return "KILLSWITCH_STATE_NO_ADAPTER";
+	case KILLSWITCH_STATE_SOFT_BLOCKED:
+		return "KILLSWITCH_STATE_SOFT_BLOCKED";
+	case KILLSWITCH_STATE_UNBLOCKED:
+		return "KILLSWITCH_STATE_UNBLOCKED";
+	case KILLSWITCH_STATE_HARD_BLOCKED:
+		return "KILLSWITCH_STATE_HARD_BLOCKED";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 static void
 update_killswitch (BluetoothKillswitch *killswitch,
 		   guint index, guint soft, guint hard)
@@ -111,7 +128,9 @@ update_killswitch (BluetoothKillswitch *killswitch,
 	}
 
 	if (changed != FALSE) {
-		g_message ("updating killswitch status %d", index);
+		g_message ("updating killswitch status %d to %s",
+			   index,
+			   state_to_string (bluetooth_killswitch_get_state (killswitch)));
 		g_signal_emit (G_OBJECT (killswitch),
 			       signals[STATE_CHANGED],
 			       0, bluetooth_killswitch_get_state (killswitch));
@@ -161,8 +180,8 @@ bluetooth_killswitch_get_state (BluetoothKillswitch *killswitch)
 	for (l = priv->killswitches ; l ; l = l->next) {
 		BluetoothIndKillswitch *ind = l->data;
 
-		g_message ("killswitch %d is %d",
-			   ind->index, ind->state);
+		g_message ("killswitch %d is %s",
+			   ind->index, state_to_string (ind->state));
 
 		if (ind->state == KILLSWITCH_STATE_HARD_BLOCKED) {
 			state = KILLSWITCH_STATE_HARD_BLOCKED;
@@ -177,7 +196,7 @@ bluetooth_killswitch_get_state (BluetoothKillswitch *killswitch)
 		state = ind->state;
 	}
 
-	g_message ("killswitches state %d", state);
+	g_message ("killswitches state %s", state_to_string (state));
 
 	return state;
 }
@@ -222,13 +241,61 @@ add_killswitch (BluetoothKillswitch *killswitch,
 	BluetoothKillswitchPrivate *priv = BLUETOOTH_KILLSWITCH_GET_PRIVATE (killswitch);
 	BluetoothIndKillswitch *ind;
 
-	g_message ("adding killswitch idx %d state %d", index, state);
+	g_message ("adding killswitch idx %d state %s", index, state_to_string (state));
 	ind = g_new0 (BluetoothIndKillswitch, 1);
 	ind->index = index;
 	ind->state = state;
 	priv->killswitches = g_list_append (priv->killswitches, ind);
 }
 
+static const char *
+type_to_string (unsigned int type)
+{
+	switch (type) {
+	case RFKILL_TYPE_ALL:
+		return "ALL";
+	case RFKILL_TYPE_WLAN:
+		return "WLAN";
+	case RFKILL_TYPE_BLUETOOTH:
+		return "BLUETOOTH";
+	case RFKILL_TYPE_UWB:
+		return "UWB";
+	case RFKILL_TYPE_WIMAX:
+		return "WIMAX";
+	case RFKILL_TYPE_WWAN:
+		return "WWAN";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static const char *
+op_to_string (unsigned int op)
+{
+	switch (op) {
+	case RFKILL_OP_ADD:
+		return "ADD";
+	case RFKILL_OP_DEL:
+		return "DEL";
+	case RFKILL_OP_CHANGE:
+		return "CHANGE";
+	case RFKILL_OP_CHANGE_ALL:
+		return "CHANGE_ALL";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static void
+print_event (struct rfkill_event *event)
+{
+	g_message ("RFKILL event: idx %u type %u (%s) op %u (%s) soft %u hard %u",
+		   event->idx,
+		   event->type, type_to_string (event->type),
+		   event->op, op_to_string (event->op),
+		   event->soft, event->hard);
+}
+
 static gboolean
 event_cb (GIOChannel *source,
 	  GIOCondition condition,
@@ -237,20 +304,23 @@ event_cb (GIOChannel *source,
 	if (condition & G_IO_IN) {
 		GIOStatus status;
 		struct rfkill_event event;
+		gsize read;
+		gboolean changed = FALSE;
 
 		status = g_io_channel_read_chars (source,
 						  (char *) &event,
 						  sizeof(event),
-						  NULL,
+						  &read,
 						  NULL);
-		if (status == G_IO_STATUS_NORMAL) {
-			g_message ("RFKILL event: idx %u type %u op %u soft %u hard %u\n",
-				   event.idx, event.type, event.op,
-				   event.soft, event.hard);
 
+		while (status == G_IO_STATUS_NORMAL && read == sizeof(event)) {
 			if (event.type != RFKILL_TYPE_BLUETOOTH &&
 			    event.type != RFKILL_TYPE_ALL)
-				return TRUE;
+				goto carry_on;
+
+			print_event (&event);
+
+			changed = TRUE;
 
 			if (event.op == RFKILL_OP_CHANGE) {
 				update_killswitch (killswitch, event.idx, event.soft, event.hard);
@@ -262,10 +332,18 @@ event_cb (GIOChannel *source,
 				add_killswitch (killswitch, event.idx, state);
 			}
 
+carry_on:
+			status = g_io_channel_read_chars (source,
+							  (char *) &event,
+							  sizeof(event),
+							  &read,
+							  NULL);
+		}
+
+		if (changed)
 			g_signal_emit (G_OBJECT (killswitch),
 				       signals[STATE_CHANGED],
 				       0, bluetooth_killswitch_get_state (killswitch));
-		}
 	} else {
 		g_message ("something else happened");
 		return FALSE;
