@@ -35,12 +35,11 @@
 #include "bluetooth-plugin-manager.h"
 #include "bluetooth-client.h"
 #include "bluetooth-client-private.h"
-#include "gconf-bridge.h"
 #include "general.h"
 #include "adapter.h"
 
-#define PREF_DIR		"/apps/bluetooth-manager"
-#define PREF_SHOW_ICON		PREF_DIR "/show_icon"
+#define SCHEMA_NAME		"org.gnome.Bluetooth"
+#define PREF_SHOW_ICON		"show-icon"
 
 static gboolean delete_callback(GtkWidget *window, GdkEvent *event,
 							gpointer user_data)
@@ -78,32 +77,22 @@ static void close_callback(GtkWidget *button, gpointer user_data)
 static void
 receive_callback (GtkWidget *item, GtkWindow *window)
 {
-	GtkWidget *dialog, *hbox, *image, *label;
+	GtkWidget *dialog;
 	const char *command = "gnome-file-share-properties";
 
 	if (!g_spawn_command_line_async(command, NULL)) {
 		/* translators:
 		 * This is the name of the preferences dialogue for gnome-user-share */
-		dialog = gtk_dialog_new_with_buttons(_("Cannot start \"Personal File Sharing\" Preferences"), window,
-						     GTK_DIALOG_NO_SEPARATOR,
-						     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
-
-		hbox = gtk_hbox_new (FALSE, 4);
-
-		image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_MENU);
-		gtk_box_pack_start(GTK_BOX (hbox), image, FALSE, FALSE, 4);
+		dialog = gtk_message_dialog_new (window,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Cannot start \"Personal File Sharing\" Preferences"));
 
 		/* translators:
 		 * This is the name of the preferences dialogue for gnome-user-share */
-		label = gtk_label_new (_("Please verify that the \"Personal File Sharing\" program is correctly installed."));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_box_pack_start(GTK_BOX (hbox), label, TRUE, TRUE, 4);
-
-		gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 2);
-
-		gtk_widget_show_all(hbox);
-		gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-		gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+			_("Please verify that the \"Personal File Sharing\" program is correctly installed."));
 
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
@@ -120,7 +109,7 @@ static void help_callback(GtkWidget *item)
 		g_printerr("Unable to launch help: %s", error->message);
 		g_error_free(error);
 	}
-}	
+}
 
 static GtkWidget *create_window(GtkWidget *notebook)
 {
@@ -129,7 +118,7 @@ static GtkWidget *create_window(GtkWidget *notebook)
 	GtkWidget *buttonbox;
 	GtkWidget *button;
 	GtkWidget *image;
-	GConfBridge *bridge;
+	GSettings *settings;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Bluetooth Preferences"));
@@ -148,10 +137,10 @@ static GtkWidget *create_window(GtkWidget *notebook)
 	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
 
 	button = gtk_check_button_new_with_mnemonic (_("_Show Bluetooth icon"));
-	bridge = gconf_bridge_get ();
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
-	gconf_bridge_bind_property_full (bridge, PREF_SHOW_ICON, G_OBJECT (button),
-					 "active", FALSE);
+	settings = g_settings_new (SCHEMA_NAME);
+	g_settings_bind (settings, PREF_SHOW_ICON, G_OBJECT (button), "active",
+			 G_SETTINGS_BIND_DEFAULT);
 
 	buttonbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox), GTK_BUTTONBOX_END);
